<?php
if ( !defined( 'IN_CMS' ) ) {
    exit();
}

use models\Template;
use models\Role;
use models\Permission;
use models\RolePermission;

$schema = \IoC::resolve( 'sqlschema' );

$schema->dropTableIfexists( 'answer' );
$schema->dropTableIfexists( 'responder' );
$schema->dropTableIfexists( 'choice' );
$schema->dropTableIfexists( 'question' );
$schema->dropTableIfexists( 'survey' );

$schema->createTable( 'survey', [
    'survey_id' => ['integer', ['not_null' => true, 'primary_key' => true, 'identity' => true]],
    'title' => ['varchar', ['length' => 255, 'not_null' => true]],
    'description' => ['text', ['default'=> 'NULL']],
    'language' => ['varchar', ['length' => 2, 'default'=> 'NULL']],
    'user_id' => ['integer', ['default' => 'NULL']],
    'status' => ['smallint', ['not_null' => true]],
    'start_date' => ['date', ['not_null' => true]],
    'start_time' => ['time', ['not_null' => true]],
    'end_date' => ['date', ['default' => 'NULL']],
    'end_time' => ['time', ['default' => 'NULL']],
    'created_at' => ['datetime', ['not_null' => true]],
    'updated_at' => ['datetime', ['default' => 'NULL']]
] );

$schema->addForeignKey( 'FK_survey', 'survey', 'user_id', 'user', 'user_id', 'SET NULL', 'CASCADE' );

$schema->createTable( 'question', [
    'question_id' => ['integer', ['not_null' => true, 'primary_key' => true, 'identity' => true]],
    'text' => ['varchar', ['length' => 255, 'not_null' => true]],
    'type' => ['varchar', ['length' => 16, 'not_null' => true]],
    'is_required' => ['boolean', ['not_null' => true]],
    'position' => ['smallint', ['not_null' => true]],
    'properties' => ['clob', ['default'=> 'NULL']],
    'survey_id' => ['integer', ['not_null' => true]],
    'created_at' => ['datetime', ['not_null' => true]],
    'updated_at' => ['datetime', ['default' => 'NULL']]
] );

$schema->addForeignKey( 'FK_question', 'question', 'survey_id', 'survey', 'survey_id', 'CASCADE', 'CASCADE' );

$schema->createTable( 'choice', [
    'choice_id' => ['integer', ['not_null' => true, 'primary_key' => true, 'identity' => true]],
    'text' => ['varchar', ['length' => 255, 'not_null' => true]],
    'question_id' => ['integer', ['not_null' => true]]
] );

$schema->addForeignKey( 'FK_choice', 'choice', 'question_id', 'question', 'question_id', 'CASCADE', 'CASCADE' );

$schema->createTable( 'responder', [
    'responder_id' => ['integer', ['not_null' => true, 'primary_key' => true, 'identity' => true]],
    'survey_id' => ['integer', ['not_null' => true]],
    'hashed_fingerprint' => ['varchar', ['length' => 50, 'not_null' => true]],
    'created_at' => ['datetime', ['not_null' => true]],
    'updated_at' => ['datetime', ['default' => 'NULL']]
] );

$schema->addIndex( 'UX_responder', 'responder', ['survey_id', 'hashed_fingerprint'], true );
$schema->addForeignKey( 'FK_responder_survey', 'responder', 'survey_id', 'survey', 'survey_id', 'CASCADE', 'CASCADE' );

$schema->createTable( 'answer', [
    'answer_id' => ['integer', ['not_null' => true, 'primary_key' => true, 'identity' => true]],
    'responder_id' => ['integer', ['not_null' => true]],
    'question_id' => ['integer', ['not_null' => true]],
    'choice_id' => ['integer', ['default' => 'NULL']],
    'value' => ['text', ['default'=> 'NULL']]
] );

$schema->addForeignKey( 'FK_answer_responder', 'answer', 'responder_id', 'responder', 'responder_id', 'CASCADE', 'CASCADE' );
$schema->addForeignKey( 'FK_answer_question', 'answer', 'question_id', 'question', 'question_id', 'CASCADE', 'CASCADE' );
$schema->addForeignKey( 'FK_answer_choice', 'answer', 'choice_id', 'choice', 'choice_id', 'CASCADE', 'CASCADE' );

$configurationHandler = \IoC::resolve( 'databaseConfigurationHandler' );
$settings = new \Configuration( $configurationHandler );

$template = new Template();
$template->name = 'Survey';
$template->content_type = 'text/html';
$template->content = file_get_contents ( dirname( __FILE__ ) . DS . TEMPLATES_DIR . 'survey' . DS . 'show.html' );
$template->save();

$settings['surveys.survey_template'] = $template->template_id;

$configurationHandler->save();

$permissions = [
    'surveys.manage_surveys'
];

$administrator = Role::findOne( ['name' => 'administrator'] );
$editor = Role::findOne( ['name' => 'editor'] );

foreach ( $permissions as $name ) {
    $permission = new Permission( ['name' => $name] );
    $permission->save();
            
    $rolePermission = new RolePermission( ['role_id' => $administrator->role_id, 'permission_id' => $permission->permission_id] );
    $rolePermission->save();
    
    $rolePermission = new RolePermission( ['role_id' => $editor->role_id, 'permission_id' => $permission->permission_id] );
    $rolePermission->save();
}

$logger = \IoC::resolve( 'logger' );
$logger->info( 'Plugin surveys successfully installed.' );

return true;
